@@ -12,18 +12,16 @@
  * Distributed under the GNU GPL v3. For full terms see the file docs/COPYING.
  *
  * @ingroup pages_issue
+ *
  * @brief Handle requests for issue functions.
  *
  */
 switch ($op) {
-	case 'index':
-	case 'current':
-	case 'archive':
-	case 'view':
-	case 'download':
-		define('HANDLER_CLASS', 'IssueHandler');
-		import('pages.issue.IssueHandler');
-		break;
+    case 'index':
+    case 'current':
+    case 'archive':
+    case 'view':
+    case 'download':
+        define('HANDLER_CLASS', 'APP\pages\issue\IssueHandler');
+        break;
 }
-
-
